
#include <assert.h>
#include <errno.h>

#include <unistd.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/mman.h>
#include <sys/param.h>
#include <sys/wait.h>
#include "file.h"
#include "ee.h"
#include "viz.h"
#include "encode.h"
#include "edit.h"
#include "keystore.h"
#include "cli.h"
#include "devurandom.h"
#include "tweetnacl.h"
#include "zeit.h"
#include "customization.h"
#include "cli_data.h"
#include "cli_osx.h"

extern key_store_t *keys;


#ifdef HAVE_OSX

int cli_attach(int argc, char **argv)
{
  char *cwd;
  char *the_argv[4];
  char path[MAXPATHLEN+1];
  int slen;

  if ((2 > argc) || (3 < argc)) {
    fprintf(stderr, "Usage: %s <file> [<nick>]\n", argv[0]);
    return 0;
  }
  cwd = getcwd(NULL, 0);

  if (NULL == cwd) {
    fprintf(stderr, "Cannot get CWD.\n");
    return -1;
  }
  slen = snprintf(path, MAXPATHLEN, "%s/%s", cwd, argv[1]);

  free(cwd);

  if ((0 > slen) || (MAXPATHLEN < slen)) {
    fprintf(stderr, "Path is too long.\n");
    return -1;
  }
  the_argv[0] = "ee_attach.sh";
  the_argv[1] = path;

  if (3 == argc) {
    the_argv[2] = argv[2];
    the_argv[3] = NULL;
  }
  else {
    the_argv[2] = NULL;
    the_argv[3] = NULL;
  }
  return cli_exec(argc, the_argv);
}


int cli_fetch(int argc, char **argv)
{
  char *cwd;
  char *the_argv[2];
  char path[MAXPATHLEN+1];
  int slen;

  if ((2 > argc) || (3 < argc)) {
    fprintf(stderr, "Usage: %s <save file> [<plaintext file>]\n", argv[0]);
    return 0;
  }
  cwd = getcwd(NULL, 0);

  if (NULL == cwd) {
    fprintf(stderr, "Cannot get CWD.\n");
    return -1;
  }
  slen = snprintf(path, MAXPATHLEN, "%s/%s", cwd, argv[1]);

  free(cwd);

  if ((0 > slen) || (MAXPATHLEN < slen)) {
    fprintf(stderr, "Path is too long.\n");
    return -1;
  }
  the_argv[0] = "ee_fetch.sh";
  the_argv[1] = path;

  if (cli_exec(2, the_argv) < 0) {
    fprintf(stderr, "Can't run %s with path %s.\n", the_argv[0], the_argv[1]);
    return 0;
  }
  if (3 == argc) {
    return cli_dec(argc, argv);
  }
  return 1;
}


int cli_view(int argc, char **argv)
{
  char *the_argv[2];

  if (3 != argc) {
    fprintf(stderr, "Usage: %s <save file> <plaintext file>\n", argv[0]);
    return 0;
  }
  if (cli_fetch(argc, argv) < 0) {
    return -1;
  }
  the_argv[0] = "less";
  the_argv[1] = argv[2];

  return cli_exec(2, the_argv);
}

#endif

