
#include <ctype.h>
#include <unistd.h>
#include "encode.h"

uint8_t base64e[64] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz@#";

uint8_t base64[256] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  // 0x20
  0x00, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  // 0x30
  0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
  0x08, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  // 0x40
  0x3e, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10,
  0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
  0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20,
  0x21, 0x22, 0x23, 0x00, 0x00, 0x00, 0x00, 0x00,
  // 0x60
  0x00, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a,
  0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32,
  0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a,
  0x3b, 0x3c, 0x3d, 0x00, 0x00, 0x00, 0x00, 0x00,
  // safety
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  // safety
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  // safety
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  // safety
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};


size_t distill64(int fd, uint8_t *buf, size_t len)
{
  ssize_t avail;
  ssize_t i;
  uint8_t line[80];
  uint8_t c;
  size_t  j;

  avail = read(fd, line, sizeof(line));
  j = 0;

  while (avail > 0) {
    for (i=0; (i<avail) && (j < len); i++) {
      c = line[i];

      if (isalnum(c) || ('@' == c) || ('#' == c)) {
	buf[j++] = c;
      }
    }
    avail = read(fd, line, sizeof(line));
  }
  return j;
}


size_t encode64(uint8_t *d, uint8_t *s, size_t len)
{
  uint32_t w;
  size_t i;

  for (i=0; len >= 3; len-=3) {
    w = *s++;
    w = (w << 8) | *s++;
    w = (w << 8) | *s++;
    *d++ = base64e[(w >> 18) & 0x3f];
    *d++ = base64e[(w >> 12) & 0x3f];
    *d++ = base64e[(w >>  6) & 0x3f];
    *d++ = base64e[(w)       & 0x3f];
    i += 4;
  }
  if (2 == len) {
    w = *s++;
    w = (w << 8) | *s++;
    *d++ = base64e[(w >> 10) & 0x3f];
    *d++ = base64e[(w >>  4) & 0x3f];
    *d++ = base64e[(w <<  2) & 0x3f];
    i += 3;
  }
  if (1 == len) {
    w = *s++;
    *d++ = base64e[(w >> 2) & 0x3f];
    *d++ = base64e[(w << 4) & 0x3f];
    i += 2;
  }
  return i;
}


size_t decode64(uint8_t *buf, size_t len)
{
  uint32_t w;
  uint8_t *p;
  size_t i;

  for (i=0, p=buf; len >= 4; len-=4) {
    w = base64[*buf++];
    w = (w << 6) | base64[*buf++];
    w = (w << 6) | base64[*buf++];
    w = (w << 6) | base64[*buf++];
    *p++ = (uint8_t)(w >> 16);
    *p++ = (uint8_t)(w >> 8);
    *p++ = (uint8_t)(w);
    i += 3;
  }
  if (3 == len) {
    w = base64[*buf++];
    w = (w << 6) | base64[*buf++];
    w = (w << 6) | base64[*buf++];
    *p++ = (uint8_t)(w >> 10);
    *p++ = (uint8_t)(w >> 2);
    i += 2;
  }
  if (2 == len) {
    w = base64[*buf++];
    w = (w << 6) | base64[*buf++];
    *p++ = (uint8_t)(w >> 4);
    i += 1;
  }
  return i;
}

