
/* (C)opyright 2015 Volker Roth.  All rights reserved.
 *
 * Prepare include/ and lib/ and compile like this
 * cc -o ab -I ../include -L ../lib -l AddressBook -l CoreFoundation ab.c
 */

#include <AddressBook/ABAddressBookC.h>

#include <string.h>
#include <unistd.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

void usage(char *name)
{
  printf("USAGE: %s <flags> <nickname>\n", name);
  printf("  -h this message\n");
  printf("  -p query for phone number instead of e-mail address\n");
  printf("  -d output diagnostic messages\n");
}


int main(int argc, char **argv)
{
  /* For argument parsing.
   */
  CFStringRef kProp;
  char *name;
  int ch;
  int b;

  kProp = kABEmailProperty;
  name = argv[0];
  b = 1;

  while ((ch = getopt(argc, argv, "hpd")) != -1) {
    switch (ch)
    {
    case 'p':
      kProp = kABPhoneProperty;
      break;

    case 'd':
      b = 0;
      break;

    case 'h':
    case '?':
      usage(name);
      exit(0);

    default:
      break;
    }
  }
  argc -= optind;
  argv += optind;

  if (argc != 1) {
    usage(name);
    exit(0);
  }
  ABSearchElementRef se;
  ABAddressBookRef ab;
  ABMultiValueRef mv;
  ABRecordRef rec;
  CFStringRef nick;
  CFStringRef pid;
  CFStringRef val;
  CFArrayRef res;
  CFIndex idx;
  CFIndex max;
  char buf[256];
  int status;
  int ok;

  /*
   * ABPersonCreateSearchElement(
   *   CFStringRef property,
   *   CFStringRef label,
   *   CFStringRef key,
   *   CFTypeRef   value,
   *   ABSearchComparison comparison
   * );
   */

  status = -1;

  nick = CFStringCreateWithCString(
    NULL,
    argv[0],
    kCFStringEncodingASCII
  );

  ab = ABGetSharedAddressBook();

  se = ABPersonCreateSearchElement(
    kABNicknameProperty,
    NULL, /* search all labels */
    NULL, /* search all dictionary keys */
    nick,
    kABEqualCaseInsensitive
  );

  res = ABCopyArrayOfMatchingRecords(ab, se);
  max = CFArrayGetCount(res);

  if (0 == max) {
    b || fprintf(stderr, "Found no results.\n");
    goto release1;
  }
  b || fprintf(stderr, "Found %ld results\n", max);
  rec = (ABRecordRef)CFArrayGetValueAtIndex(res, 0);
  mv  = (ABMultiValueRef)ABRecordCopyValue(rec, kProp);

  if (NULL == mv) {
    b || fprintf(stderr, "Can't get the value I want.\n");
    goto release1;
  }
  pid = ABMultiValueCopyPrimaryIdentifier(mv);
  idx = ABMultiValueIndexForIdentifier(mv, pid);
  val = ABMultiValueCopyValueAtIndex(mv, idx);

  ok  = CFStringGetCString(val, buf, sizeof(buf), kCFStringEncodingASCII);

  if (!ok) {
    b || fprintf(stderr, "Can't convert CFString.\n");
    goto release2;
  }
  printf("%s\n", buf);

  status = 1;

 release2:
  CFRelease(mv);
  CFRelease(pid);
  CFRelease(val);

 release1:
  CFRelease(nick);
  CFRelease(se);
  CFRelease(res);

  return status;
}
