
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "encode.h"
#include "file.h"
#include "viz.h"
#include "tweetnacl.h"

#define WD 18
#define HT 8

uint8_t rnd[] = {
  0xfd, 0x6f, 0xe7, 0xb6, 0xe0, 0xcd, 0xae, 0x83,
  0x09, 0x98, 0x98, 0x94, 0x6c, 0xa6, 0x70, 0xc2,
  0xc7, 0x07, 0x37, 0x3c, 0xf1, 0xc9, 0x34, 0xad,
  0xe5, 0x12, 0x0b, 0xea, 0xa6, 0x50, 0x45, 0x22
};

uint8_t alpha[] = "0123456789abcdefghi:k-m\"opqrstuw.@";
int     hash_bits;
uint8_t hash_key[40]; /* We need 33, this is padded to multiple of 8 */


uint8_t pseudornd(int bits)
{
  uint8_t digest[64];

  hash_bits += bits;
  hash_key[32]++;
  crypto_hash(digest, hash_key, 33L);  
  return digest[0];
}


void hash_fill(canvas_t *canvas)
{
  int x;
  int y;
  int v;
  int w;
  uint8_t *p;
  uint8_t *q;
  uint8_t c;
  uint8_t r;
  uint8_t o;

  r = pseudornd(1) & 1;

  for (v=0, y=1; y < canvas->ht; y+=3, v++) {
    p = &canvas->iso[v * canvas->mod];
    q = p;
    o = r;

    for (x=1; x < canvas->wd; x+=3) {
      w = canvas_encode(canvas, x, y);
      c = canvas->code[w].c;

      if (c >= 0x80) {
	*p++ = 0xe2;
	*p++ = 0x94;
	*p++ = c;

	if (5 == w) {
	  o ^= 1;
	}
	if ((7 == w) && (x == 1)) {
	  r ^= 1;
	}
	continue;
      }
      if (('+' == c) || ('|' == c) || ('-' == c)) {
	*p++ = c;

	if (5 == w) {
	  o ^= 1;
	}
	if ((7 == w) && (x == 1)) {
	  r ^= 1;
	}
	continue;
      }
      if ((1 == o) && (160 >= hash_bits)) {
	w = pseudornd(5) % (sizeof(alpha) - 1);
	*p++ = alpha[w];
      }
      else {
	*p++ = 0x20;
      }
    }
    *p = 0;
    
    printf("%s\n", q);
  }
}


void hash_disect(canvas_t *canvas)
{
  int x;
  int y;
  int u;
  int v;
  int a;
  int b;
  int r;
  int s;
  uint8_t c;
  int i;

  r = (canvas->wd / 3) - 1;
  s = (canvas->ht / 3) - 1;
  x = 0;
  y = 0;
  u = r;
  v = s;

  canvas_line(canvas, 0, 0, 0, s);
  canvas_line(canvas, 0, 0, r, 0);
  canvas_line(canvas, r, s, 0, s);
  canvas_line(canvas, r, s, r, 0);

  for (i=0; i<5; i++) {
    c = pseudornd(6);
    a = (u - x) * (c & 7);
    b = (v - y) * ((c >> 3) & 7);
    a >>= 3;
    b >>= 3;
    a = x + a;
    b = y + b;

    if (i > 3) {
      canvas_line(canvas, 0, b, r, b);
      canvas_line(canvas, a, 0, a, s);
    }
    else {
      canvas_line(canvas, x, b, u, b);
      canvas_line(canvas, a, y, a, v);
    }
    if ((a - x) > (u - a)) {
      u = a;
    }
    else {
      x = a;
    }
    if ((b - y) > (v - b)) {
      v = b;
    }
    else {
      y = b;
    }
  }
}


void hash_print(uint8_t key[32], int type)
{
  canvas_t canvas;

  memcpy(hash_key, key, 32L);
  hash_key[32] = 0;
  hash_bits = 0;

  canvas_init(&canvas, WD, HT, type);
  hash_disect(&canvas);
  hash_fill(&canvas);
  printf("Security: %u bits\n", hash_bits);
  canvas_free(&canvas);
}


void usage(char *name)
{
  printf("USAGE: %s <flags> <file>\n", name);
  printf("  -h this message\n");
  printf("  -a output ascii instead of semigraphics\n");
  printf("  -k output key\n");
}


int main(int argc, char **argv)
{
  /* For argument parsing.
   */
  char *name;
  int type;
  int key;
  int ch;

  name = argv[0];
  type = 0;
  key  = 0;

  while ((ch = getopt(argc, argv, "hak")) != -1) {
    switch (ch)
    {
    case 'a':
      type = 1;
      break;

    case 'k':
      key = 1;
      break;

    case 'h':
    case '?':
      usage(name);
      exit(0);

    default:
      break;
    }
  }
  argc -= optind;
  argv += optind;

  if (argc != 1) {
    usage(name);
    exit(0);
  }
  (void)file_loadsync(argv[0], rnd, 32L);

  uint8_t buf[43];

  if (0 == key) {
    hash_print(rnd, type);
  }
  else {
    encode64(buf, rnd, 32L);
    key_print(STDOUT_FILENO, buf, 43L, type, 1);
  }
}
